#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import packages
from persistent import Persistent
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.copy import clone
from zope.interface import alsoProvides, implementer, noLongerProvides
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.interfaces import ITraversable

# import interfaces
from pyams_portal.interfaces import ILocalTemplateHandler, IPortalContext, IPortalPage, IPortalPortletsConfiguration, \
    IPortalTemplate, IPortalTemplateConfiguration, LOCAL_TEMPLATE_NAME, PORTAL_PAGE_KEY, PORTLETS_CONFIGURATION_KEY
from pyams_portal.portlet import PortalPortletsConfiguration
from pyams_portal.template import PortalTemplate
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.registry import query_utility
from pyams_utils.zodb import volatile_property


@implementer(IPortalPage)
class PortalPage(Persistent, Contained):
    """Portal page persistent class

    The page is the highest configuration level.
    It defines which template is used (a shared or local one), which gives
    the slot and portlet lists.
    """

    _inherit_parent = FieldProperty(IPortalPage['inherit_parent'])
    _use_local_template = FieldProperty(IPortalPage['use_local_template'])
    _local_template = FieldProperty(IPortalPage['local_template'])
    _shared_template = FieldProperty(IPortalPage['shared_template'])

    @volatile_property
    def can_inherit(self):
        return IPortalContext.providedBy(self.__parent__.__parent__)

    @property
    def inherit_parent(self):
        return self._inherit_parent if self.can_inherit else False

    @inherit_parent.setter
    def inherit_parent(self, value):
        self._inherit_parent = value if self.can_inherit else False

    @property
    def override_parent(self):
        return not self.inherit_parent

    @override_parent.setter
    def override_parent(self, value):
        self.inherit_parent = not value

    @property
    def parent(self):
        parent = self.__parent__
        page = IPortalPage(parent)
        while page.inherit_parent:
            parent = parent.__parent__
            page = IPortalPage(parent)
        return parent

    @property
    def use_local_template(self):
        return False if self.inherit_parent else self._use_local_template

    @use_local_template.setter
    def use_local_template(self, value):
        self._use_local_template = value
        if value and (self._local_template is None) and not self.inherit_parent:
            registry = get_current_registry()
            template = PortalTemplate()
            template.name = LOCAL_TEMPLATE_NAME
            self._local_template = template
            registry.notify(ObjectCreatedEvent(template))
            locate(template, self, '++template++')
        if self.use_local_template:
            alsoProvides(self, ILocalTemplateHandler)
        else:
            noLongerProvides(self, ILocalTemplateHandler)

    @property
    def local_template(self):
        return self._local_template

    @property
    def use_shared_template(self):
        return IPortalPage(self.parent).use_shared_template if self.inherit_parent else not self._use_local_template

    @use_shared_template.setter
    def use_shared_template(self, value):
        self.use_local_template = not value

    @property
    def shared_template(self):
        return IPortalPage(self.parent).shared_template if self.inherit_parent else self._shared_template

    @shared_template.setter
    def shared_template(self, value):
        if not self.inherit_parent:
            if IPortalTemplate.providedBy(value):
                value = value.name
            self._shared_template = value

    @property
    def template(self):
        if self.inherit_parent:
            template = IPortalPage(self.parent).template
        else:
            if self.use_local_template:
                template = self.local_template
            else:
                template = self.shared_template
        if isinstance(template, str):
            template = query_utility(IPortalTemplate, name=template)
        return template


@adapter_config(context=IPortalContext, provides=IPortalPage)
def portal_context_page_adapter(context):
    """Portal context page factory"""
    return get_annotation_adapter(context, PORTAL_PAGE_KEY, PortalPage)


@adapter_config(name='template', context=IPortalContext, provides=ITraversable)
class PortalContextTemplateTraverser(ContextAdapter):
    """++template++ portal context traverser"""

    def traverse(self, name, furtherpath=None):
        return IPortalPage(self.context).local_template


@adapter_config(context=IPortalContext, provides=IPortalTemplateConfiguration)
def portal_context_template_configuration_adapter(context):
    """Portal context template configuration adapter"""
    template = IPortalPage(context).template
    return IPortalTemplateConfiguration(template)


@adapter_config(context=IPortalContext, provides=IPortalPortletsConfiguration)
def portal_context_portlets_configuration_adapter(context):
    """Portal context portlets configuration adapter"""

    def portlet_configuration_factory():
        return PortalPortletsConfiguration.clone(portlets_config, context)

    # get page and template
    page = IPortalPage(context)
    if page.use_local_template:
        template = page.local_template
    else:
        template = page.template
    if template is not None:
        portlets_config = IPortalPortletsConfiguration(template)
        if page.use_local_template:
            context = template
        # get current configuration
        config = get_annotation_adapter(context, PORTLETS_CONFIGURATION_KEY,
                                        factory=portlet_configuration_factory,
                                        notify=False, locate=False)
        # check for missing portlets configuration
        for portlet_id, portlet_config in portlets_config.items():
            if portlet_id not in config:
                config.set_portlet_configuration(portlet_id, clone(portlet_config))
        return config


@adapter_config(name='portlet', context=IPortalContext, provides=ITraversable)
class PortalContextPortletTraverser(ContextAdapter):
    """++portlet++ portal context traverser"""

    def traverse(self, name, thurtherpath=None):
        config = IPortalPortletsConfiguration(self.context)
        if name:
            return config.get_portlet_configuration(int(name))
        else:
            return config
